from __future__ import annotations

from .AFXProcedure import AFXProcedure


class AFXStep:
    """Abaqus."""

    def __init__(self, prompt: str, owner: AFXProcedure):
        """Constructor.

        Parameters
        ----------
        prompt : str
            Prompt.
        owner : AFXProcedure
            Owner.
        """

    def onCancel(self):
        """Called when the step is cancelled.

        Reimplemented in AFXCreateSketchStep, AFXDialogStep, AFXEditSketchStep, AFXOrderedPickStep, and
        AFXPickStep.
        """

    def onDone(self):
        """Called when the step completes."""

    def onExecute(self):
        """Called to execute the steps returned by getFirstStep and getNextStep.

        Reimplemented in AFXCreateSketchStep, AFXDialogStep, AFXEditSketchStep, AFXOrderedPickStep, and
        AFXPickStep.
        """

    def onResume(self):
        """Called when the step is resumed.

        Reimplemented in AFXCreateSketchStep, AFXDialogStep, AFXEditSketchStep, and AFXPickStep.
        """

    def onSuspend(self):
        """Called when the step is suspended.

        Reimplemented in AFXCreateSketchStep, AFXDialogStep, AFXEditSketchStep, and AFXPickStep.
        """

    def onValueChanged(self):
        """Called when the step's value changes."""

    def reset(self):
        """Allows a step to reset any of its data (if needed) when looping.

        Reimplemented in AFXOrderedPickStep, and AFXPickStep. By clicking on Send, you accept that Dassault
        Systèmes will process your personal data and may contact you for further information. [Privacy
        Policy](
        https://www.3ds.com/privacy-policy).
        Total Results: Results per page This page cannot be found. The page might not exist or is
        temporarily unavailable. Try again or try searching for the topic. Use this form to provide feedback
        on this help topic. To get product support or to provide product feedback, go to [Frequently Asked
        Questions](
        https://3ds.one/PO).
        For support for online purchased solutions, go to [Online Purchase Support](https://3ds.one/Q8). *
        Required Subject: Feedback on User Assistance * I acknowledge I have read and I hereby accept the
        [privacy policy](
        https://www.3ds.com/privacy-policy)
        under which my personal data will be used by Dassault Systèmes. Thank you for your comments. We will contact you if we have questions regarding your feedback. Sincerely, The Dassault Systèmes User Assistance Team Dialog for Cookie Allow to use Cookie for saving your settings ? Drag to outliner or Upload Close
        """



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc

from ..UtilityAndView.abaqusConstants import ON, Boolean, SymbolicConstant
from .InteractionState import InteractionState


@abaqus_class_doc
class XFEMCrackGrowthState(InteractionState):
    """The XFEMCrackGrowthState object stores the propagating data of an XFEMCrackGrowth object in a step. One
    instance of this object is created internally by the XFEMCrackGrowth object for each step. The instance is
    also deleted internally by the XFEMCrackGrowth object. The XFEMCrackGrowthState object has no constructor or
    methods. The XFEMCrackGrowthState object is derived from the InteractionState object.

    .. note::
        This object can be accessed by::

            import interaction
            mdb.models[name].steps[name].interactionStates[name]

        The corresponding analysis keywords are:

        - ENRICHMENT ACTIVATION
    """

    #: A Boolean specifying whether the crack is allowed to grow (propagate) during this
    #: analysis step. The default value is ON.
    allowGrowth: Boolean = ON

    #: A SymbolicConstant specifying the propagation state of the **allowGrowth** member.
    #: Possible values are UNSET, SET, UNCHANGED, and FREED.
    allowGrowthState: SymbolicConstant

    #: A SymbolicConstant specifying the propagation state of the InteractionState object.
    #: Possible values are:
    #:
    #: - NOT_YET_ACTIVE
    #: - CREATED
    #: - PROPAGATED
    #: - MODIFIED
    #: - DEACTIVATED
    #: - NO_LONGER_ACTIVE
    #: - TYPE_NOT_APPLICABLE
    #: - INSTANCE_NOT_APPLICABLE
    #: - BUILT_INTO_BASE_STATE
    status: SymbolicConstant



from __future__ import annotations

from .constants import DEFAULT_SPACING
from .FXComposite import FXComposite
from .FXPacker import FXPacker


class FXVerticalFrame(FXPacker):
    """Vertical frame layout manager widget is used to automatically place child-windows vertically from top-to-
    bottom, or bottom-to-top, depending on the child window's layout hints."""

    def __init__(
        self,
        p: FXComposite,
        opts: int = 0,
        x: int = 0,
        y: int = 0,
        w: int = 0,
        h: int = 0,
        pl: int = DEFAULT_SPACING,
        pr: int = DEFAULT_SPACING,
        pt: int = DEFAULT_SPACING,
        pb: int = DEFAULT_SPACING,
        hs: int = DEFAULT_SPACING,
        vs: int = DEFAULT_SPACING,
    ):
        """Construct a vertical frame layout manager.

        Parameters
        ----------
        p : FXComposite

        opts : int

        x : int

        y : int

        w : int

        h : int

        pl : int

        pr : int

        pt : int

        pb : int

        hs : int

        vs : int
        """

    def getDefaultHeight(self):
        """Return default height.

        Reimplemented from FXPacker. Reimplemented in AFXVerticalAligner.
        """

    def getDefaultWidth(self):
        """Return default width.

        Reimplemented from FXPacker. Reimplemented in AFXVerticalAligner.
        """



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..Assembly.PartInstance import PartInstance
from ..BasicGeometry.ModelDotArray import ModelDotArray
from ..UtilityAndView.abaqusConstants import OFF, Boolean
from .Constraint import Constraint


@abaqus_class_doc
class DisplayBody(Constraint):
    """The DisplayBody object defines a constraint such that the specified instance is used for display only and
    does not take part in the analysis. However it will still be visible during postprocessing and its position
    at any frame will be defined by the translation and rotation of the specified control points. The
    DisplayBody object is derived from the ConstrainedSketchConstraint object.

    .. note::
        This object can be accessed by::

            import interaction
            mdb.models[name].constraints[name]

        The corresponding analysis keywords are:

        - DISPLAY BODY
    """

    #: A Boolean specifying whether the constraint is suppressed or not. The default value is
    #: OFF.
    suppressed: Boolean = OFF

    #: A String specifying the constraint repository key.
    name: str

    #: A PartInstance object specifying the part instance that is to be used for display only.
    instance: PartInstance

    #: A ModelDotArray object specifying the motion of the PartInstance. The control points may
    #: be ConstrainedSketchVertex, ReferencePoint, or MeshNode objects. Their motion will control the motion of
    #: the PartInstance. If this argument is set to an empty sequence, the PartInstance will
    #: remain fixed in space during the analysis. The sequence can have either one object or
    #: three objects.
    controlPoints: ModelDotArray

    @abaqus_method_doc
    def __init__(self, name: str, instance: PartInstance, controlPoints: ModelDotArray):
        """This method creates a DisplayBody object.

        .. note::
            This function can be accessed by::

                mdb.models[name].DisplayBody

        Parameters
        ----------
        name
            A String specifying the constraint repository key.
        instance
            A PartInstance object specifying the part instance that is to be used for display only.
        controlPoints
            A ModelDotArray object specifying the motion of the PartInstance. The control points may
            be ConstrainedSketchVertex, ReferencePoint, or MeshNode objects. Their motion will control the motion of
            the PartInstance. If this argument is set to an empty sequence, the PartInstance will
            remain fixed in space during the analysis. The sequence can have either one object or
            three objects.

        Returns
        -------
        DisplayBody
            A DisplayBody object.
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the DisplayBody object."""
        ...



from __future__ import annotations

from .FXObject import FXObject

#: Extended selection mode.
ICONLIST_EXTENDEDSELECT: int = hash("ICONLIST_EXTENDEDSELECT")

#: At most one selected item.
ICONLIST_SINGLESELECT: int = hash("ICONLIST_SINGLESELECT")

#: Always exactly one selected item.
ICONLIST_BROWSESELECT: int = hash("ICONLIST_BROWSESELECT")

#: Multiple selection mode.
ICONLIST_MULTIPLESELECT: int = hash("ICONLIST_MULTIPLESELECT")

#: Automatically size item spacing.
ICONLIST_AUTOSIZE: int = hash("ICONLIST_AUTOSIZE")

#: List mode.
ICONLIST_DETAILED: int = hash("ICONLIST_DETAILED")

#: Mini Icon mode.
ICONLIST_MINI_ICONS: int = hash("ICONLIST_MINI_ICONS")

#: Big Icon mode.
ICONLIST_BIG_ICONS: int = hash("ICONLIST_BIG_ICONS")

#: Row-wise mode.
ICONLIST_ROWS: int = hash("ICONLIST_ROWS")

#: Column-wise mode.
ICONLIST_COLUMNS: int = hash("ICONLIST_COLUMNS")


class FXIconItem(FXObject):
    """Icon item."""



from __future__ import annotations

from .AFXComTableKeyword import AFXComTableKeyword
from .AFXGuiCommand import AFXGuiCommand


class AFXTableKeyword(AFXComTableKeyword):
    """This class is designed for command keywords that have table values."""

    def __init__(
        self,
        command: AFXGuiCommand,
        name: str,
        isRequired: bool = False,
        minLength: int = 0,
        maxLength: int = -1,
        opts: int = 0,
    ):
        """Constructor.

        Parameters
        ----------
        command : AFXGuiCommand
            Host command.
        name : str
            Keyword name.
        isRequired : bool
            True if this keyword is a required argument.
        minLength : int
            Minimum (and default) row length.
        maxLength : int
            Maximum row length (-1 => unlimited).
        opts : int
            Options.
        """

    def getTypeName(self):
        """Returns the name of the table keyword type.

        Reimplemented from AFXComTableKeyword.
        """



from __future__ import annotations

from typing import List

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ...UtilityAndView.abaqusConstants import Boolean
from .ConstrainedSketchGeometry import ConstrainedSketchGeometry


@abaqus_class_doc
class ConstrainedSketchGeometryArray(List[ConstrainedSketchGeometry]):
    """The ConstrainedSketchGeometryArray is a sequence of ConstrainedSketchGeometry objects.

    .. note::
        This object can be accessed by::

            import sketch
            mdb.models[name].sketches[name].geometry[i]
    """

    @abaqus_method_doc
    def findAt(self, coordinates: tuple[float, float], printWarning: Boolean = True) -> ConstrainedSketchGeometry:
        """This method returns the ConstrainedSketchGeometry object located at the given coordinates.

        Parameters
        ----------
        coordinates
            A sequence of Floats specifying the **X**  and **Y** coordinates of the object to find.
        printWarning
            A Boolean specifying whether a message is to be printed to the CLI if no entity is found
            at the specified location. The default value is True.

        Returns
        -------
        ConstrainedSketchGeometry
            A ConstrainedSketchGeometry object.
        """
        return ConstrainedSketchGeometry()



from __future__ import annotations

from .GUIPlugInCommands import GUIPluginToolset
from .KernelPlugInCommands import KernelPluginToolset


class PluginToolset(GUIPluginToolset, KernelPluginToolset): ...



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc


@abaqus_class_doc
class IgnoredVertex:
    """An IgnoredVertex object is a point region of the geometry that was abstracted away by a virtual topology
    feature.

    .. note::
        This object can be accessed by::

            import part
            mdb.models[name].parts[name].ignoredVertices[i]
            import assembly
            mdb.models[name].rootAssembly.allInstances[name].ignoredVertices[i]
            mdb.models[name].rootAssembly.instances[name].ignoredVertices[i]
    """

    #: An Int specifying the index of the IgnoredVertex in the IgnoredVertexArray.
    index: int | None = None

    #: A tuple of Floats specifying the **X**, **Y**, and **Z** coordinates of the vertex.
    pointOn: tuple[float, ...] = ()



from __future__ import annotations

from .constants import DEFAULT_PAD
from .FXComposite import FXComposite
from .FXLabel import FXLabel
from .FXObject import FXObject

#: Automatically gray out when not updated.
RADIOBUTTON_AUTOGRAY: int = hash("RADIOBUTTON_AUTOGRAY")

#: Automatically hide when not updated.
RADIOBUTTON_AUTOHIDE: int = hash("RADIOBUTTON_AUTOHIDE")

#: Normal radio button style.
RADIOBUTTON_NORMAL: int = hash("RADIOBUTTON_NORMAL")


class FXRadioButton(FXLabel):
    """A radio button is a tri-state button.

    Normally, it is either True or False; a third state MAYBE may be set to indicate that no selection has
    been made yet by the user, or that the state is ambiguous. When pressed, the radio button sets its state
    to True and sends a SEL_COMMAND to its target, and the message data set to the state of the radio
    button, of the type FXbool. If the radio button is contained inside a group box, the other radio buttons
    in the group box will be set to False and will send a message as well.
    """

    def __init__(
        self,
        p: FXComposite,
        text: str,
        tgt: FXObject | None = None,
        sel: int = 0,
        opts: int = RADIOBUTTON_NORMAL,
        x: int = 0,
        y: int = 0,
        w: int = 0,
        h: int = 0,
        pl: int = DEFAULT_PAD,
        pr: int = DEFAULT_PAD,
        pt: int = DEFAULT_PAD,
        pb: int = DEFAULT_PAD,
    ):
        """Construct new radio button.

        Parameters
        ----------
        p : FXComposite

        text : str

        tgt : FXObject | None

        sel : int

        opts : int

        x : int

        y : int

        w : int

        h : int

        pl : int

        pr : int

        pt : int

        pb : int
        """

    def canFocus(self):
        """Returns True because a radio button can receive focus.

        Reimplemented from FXWindow.
        """

    def getCheck(self):
        """Get radio button state (True, False or MAYBE)."""

    def getDefaultHeight(self):
        """Get default height.

        Reimplemented from FXLabel.
        """

    def getDefaultWidth(self):
        """Get default width.

        Reimplemented from FXLabel.
        """

    def setCheck(self, s: bool = True):
        """Set radio button state (True, False or MAYBE).

        Parameters
        ----------
        s : bool
        """



from __future__ import annotations

from .FXComposite import FXComposite
from .FXIcon import FXIcon
from .FXMenuCaption import FXMenuCaption
from .FXPopup import FXPopup


class FXMenuTitle(FXMenuCaption):
    """A menu title is a child of a menu bar which is responsible for popping up a pulldown menu."""

    def __init__(self, p: FXComposite, text: str, ic: FXIcon | None = None, pup: FXPopup | None = None, opts: int = 0):
        """Constructor.

        Parameters
        ----------
        p : FXComposite

        text : str

        ic : FXIcon | None

        pup : FXPopup | None

        opts : int
        """

    def canFocus(self):
        """Yes it can receive the focus.

        Reimplemented from FXWindow.
        """

    def create(self):
        """Create server-side resources.

        Reimplemented from FXMenuCaption.
        """

    def detach(self):
        """Detach server-side resources.

        Reimplemented from FXMenuCaption.
        """

    def getDefaultHeight(self):
        """Return default height.

        Reimplemented from FXMenuCaption.
        """

    def getDefaultWidth(self):
        """Return default width.

        Reimplemented from FXMenuCaption.
        """

    def getMenu(self):
        """Return popup menu."""

    def setMenu(self, menu: FXPopup):
        """Set popup menu to pop up.

        Parameters
        ----------
        menu : FXPopup
        """



from __future__ import annotations

from .FXComposite import FXComposite
from .FXFrame import FXFrame

#: Nothing visible.
SEPARATOR_NONE: int = hash("SEPARATOR_NONE")

#: Etched-in looking groove.
SEPARATOR_GROOVE: int = hash("SEPARATOR_GROOVE")

#: Embossed looking ridge.
SEPARATOR_RIDGE: int = hash("SEPARATOR_RIDGE")

#: Simple line.
SEPARATOR_LINE: int = hash("SEPARATOR_LINE")


class FXHorizontalSeparator(FXFrame):
    """Horizontal separator."""

    def __init__(
        self,
        p: FXComposite,
        x: int = 0,
        y: int = 0,
        w: int = 0,
        h: int = 0,
        pl: int = 1,
        pr: int = 1,
        pt: int = 0,
        pb: int = 0,
    ):
        """Constructor.

        Parameters
        ----------
        p : FXComposite

        x : int

        y : int

        w : int

        h : int

        pl : int

        pr : int

        pt : int

        pb : int
        """

    def getDefaultHeight(self):
        """Return default height.

        Reimplemented from FXFrame.
        """

    def getDefaultWidth(self):
        """Return default width.

        Reimplemented from FXFrame.
        """



import re
from functools import partial
from typing import Tuple

from . import __version__ as version


def class_or_module_link(
    type: str,
    class_or_module_name: str,
    prefix="",
    suffix="",
    label=None,
) -> Tuple[str, str]:
    """Generate a link to the Abaqus class documentation.

    Parameters
    ----------
    type : str
        Type of the object, 'class' or 'module'.
    class_or_module_name : str
        The name of the class or module.
    prefix : str
        The prefix to the class or module name.
    suffix : str
        The suffix to the class name.
    label : str
        The label to use for the link.

    Returns
    -------
    str
        The link to the documentation.
    str
        The link with label.
    """
    if label is None:
        label = class_or_module_name
    module_name = f"{type if type == 'module' else ''}{class_or_module_name.lower()}"
    link = (
        f"https://help.3ds.com/{version[:4]}/English/DSSIMULIA_Established/SIMACAEKERRefMap/"
        f"simaker-c-{prefix}{module_name}{suffix}pyc.htm?contextscope=all"
    )
    return link, f"Check `{label} on help.3ds.com/{version[:4]} <{link}>`__."


def method_or_function_link(
    type: str,
    class_or_module_name: str,
    method_or_function_name: str,
    prefix="",
    suffix="",
    label=None,
) -> Tuple[str, str]:
    """Generate a link to the Abaqus function documentation.

    Parameters
    ----------
    type : str
        Type of the object, 'method' or 'function'.
    class_or_module_name : str
        The name of the module.
    method_or_function_name : str
        The name of the function.
    prefix : str
        The prefix to the class or module name.
    suffix : str
        The suffix to the class name.
    label : str
        The label to use for the link.

    Returns
    -------
    str
        The link to the documentation.
    str
        The link with label.
    """
    # For methods that start with a capital letter, the link is to the corresponding class documentation.
    if method_or_function_name[0].isupper():
        return class_or_module_link("class", method_or_function_name, prefix, suffix, method_or_function_name)
    class_name = f"{type if type == 'function' else ''}{class_or_module_name.lower()}"
    function_prefix = prefix if type == "function" else ""
    if type == "class" and method_or_function_name == "__init__":
        method_or_function_name = class_or_module_name
    link = (
        f"https://help.3ds.com/{version[:4]}/English/DSSIMULIA_Established/SIMACAEKERRefMap/"
        f"simaker-c-{prefix}{class_name}{suffix}pyc.htm?contextscope=all"
        f"#simaker-{function_prefix}{class_name}{method_or_function_name.lower()}pyc"
    )
    if method_or_function_name == class_or_module_name and type == "class":
        signature = f"{class_or_module_name}()"
    else:
        signature = f"{class_or_module_name}.{method_or_function_name}()"
    if label is None:
        label = signature
    return link, f"Check `{label} on help.3ds.com/{version[:4]} <{link}>`__."


def add_link_in_class_or_module_docstring(
    type: str,
    class_or_module_name: str,
    docstring: str,
    prefix="",
    suffix="",
    label=None,
) -> str:
    """Add a link to the Abaqus documentation to the docstring for classes or modules.

    Parameters
    ----------
    type : str
        Type of the object, 'class' or 'module'.
    class_or_module_name : str
        The name of the class or module.
    docstring : str
        The docstring.
    prefix : str
        The prefix to the class or module name.
    suffix : str
        The suffix to the class name.
    label : str
        The label to use for the link.

    Returns
    -------
    str
        The docstring with the link to the Abaqus documentation.
    """
    if not docstring:
        return docstring
    link, link_with_label = class_or_module_link(type, class_or_module_name, prefix, suffix, label)
    return (
        docstring.rstrip()
        + "\n\n"
        + " " * (0 if type == "module" else 4)
        + ".. note::\n"
        + " " * (4 if type == "module" else 8)
        + link_with_label
    )


add_link_in_class_docstring = partial(add_link_in_class_or_module_docstring, "class")
add_link_in_module_docstring = partial(add_link_in_class_or_module_docstring, "module")


def add_link_in_method_or_function_docstring(
    type: str,
    class_or_module_name: str,
    method_or_function_name: str,
    docstring: str,
    prefix="",
    suffix="",
    label=None,
) -> str:
    """Add a link to the Abaqus documentation to the docstring for methods or functions.

    Parameters
    ----------
    type : str
        Type of the object, 'method' or 'function'.
    class_or_module_name : str
        The name of the module.
    method_or_function_name : str
        The name of the function.
    docstring : str
        The docstring.
    prefix : str
        The prefix to the class or module name.
    suffix : str
        The suffix to the class name.
    label : str
        The label to use for the link.

    Returns
    -------
    str
        The docstring with the link to the Abaqus documentation.
    """
    if not docstring:
        return docstring
    link, link_with_label = method_or_function_link(
        type, class_or_module_name, method_or_function_name, prefix, suffix, label
    )
    return re.sub(
        r"(\n\s+?)(Parameters\n\s+----------)",
        r"\1" + ".. note::\n" + " " * (8 if type == "function" else 12) + link_with_label + r"\1\2",
        docstring,
    )


add_link_in_method_docstring = partial(add_link_in_method_or_function_docstring, "method")
add_link_in_function_docstring = partial(add_link_in_method_or_function_docstring, "function")


def abaqus_function_doc(func):
    """Add a link to the Abaqus documentation to the docstring of the function."""
    module_name = func.__module__.split(".")[-1]
    func.__doc__ = add_link_in_function_docstring(
        class_or_module_name=module_name,
        method_or_function_name=func.__name__,
        docstring=func.__doc__,
        prefix="gpr" if module_name.lower().startswith("cae") else "",
        label=f"{module_name}.{func.__name__}",
    )
    return func


def _process_class_name(class_name: str) -> str:
    """Process the class name."""
    # Handle the base class case.
    if class_name.endswith("Base"):
        class_name = class_name[:-4]
    # Handle the class with a suffix.
    if class_name.endswith("Mdb"):
        class_name = "Mdb"
    if class_name.endswith("Model"):
        class_name = "Model"
    if class_name.endswith("Step"):
        class_name = "Step"
    if class_name.endswith("Assembly"):
        class_name = "Assembly"
    if class_name.endswith("Step"):
        class_name = "Step"
    if class_name.endswith("Odb"):
        class_name = "Odb"
    if class_name.endswith("Part"):
        class_name = "Part"
    return class_name


class_suffix = {
    "AdaptivityModel": "kar",
    "AssemblyModel": "asm",
    "EditMeshAssembly": "edt",
    "EditMeshPart": "edt",
    "InteractionModel": "itn",
    "MeshAssembly": "mgn",
    "MeshPart": "mgn",
    "RegionAssembly": "reg",
    "RegionPart": "reg",
}


def abaqus_method_doc(method):
    """Add a link to the Abaqus documentation to the docstring of the method."""
    if method.__name__ == "__init__":
        return method
    class_name = method.__qualname__.split(".")[0]
    method.__doc__ = add_link_in_method_docstring(
        class_or_module_name=_process_class_name(class_name),
        method_or_function_name=method.__name__,
        docstring=method.__doc__,
        prefix="gpr" if class_name.lower().startswith("cae") else "",
        suffix=class_suffix.get(class_name, ""),
        label=f"{class_name}.{method.__name__}",
    )
    return method


def abaqus_class_doc(cls):
    """Add a link to the Abaqus documentation to the docstring of the class."""
    class_name = cls.__name__
    processed_class_name = _process_class_name(class_name)
    cls.__doc__ = add_link_in_class_docstring(
        class_or_module_name=processed_class_name,
        docstring=cls.__doc__,
        prefix="gpr" if class_name.lower().startswith("cae") else "",
        suffix=class_suffix.get(class_name, ""),
        label=class_name,
    )
    return cls



from __future__ import annotations

from typing import Sequence, Union, overload

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..UtilityAndView.abaqusConstants import OFF, Boolean, SymbolicConstant
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from .FieldLocation import FieldLocation
from .FieldLocationArray import FieldLocationArray
from .FieldValueArray import FieldValueArray
from .OdbInstance import OdbInstance
from .OdbMeshElement import OdbMeshElement
from .OdbMeshNode import OdbMeshNode
from .OdbSet import OdbSet
from .SectionPoint import SectionPoint


@abaqus_class_doc
class FieldOutput:
    """A FieldOutput object contains field data for a specific output variable.

    .. note::
        This object can be accessed by::

            import odbAccess
            session.odbs[name].steps[name].frames[i].fieldOutputs[name]
    """

    #: An Int specifying the dimension of vector or the first dimension (number of rows) of
    #: matrix.
    dim: int | None = None

    #: An Int specifying the second dimension (number of columns) of matrix.
    dim2: int | None = None

    #: A Boolean specifying whether the data are complex.
    isComplex: Boolean = OFF

    #: A FieldLocationArray object.
    locations: FieldLocationArray = []

    #: A FieldValueArray object specifying the order of the objects in the array is determined
    #: by the Abaqus Scripting Interface; see the **data** argument to the addData method for a
    #: description of the order.
    values: FieldValueArray | None = None

    #: A String specifying the output variable name.
    name: str

    #: A String specifying the output variable. Colon (:) should not be used as a part of the
    #: field output description.
    description: str

    #: A SymbolicConstant specifying the output type. Possible values are SCALAR, VECTOR,
    #: TENSOR_3D_FULL, TENSOR_3D_PLANAR, TENSOR_3D_SURFACE, TENSOR_2D_PLANAR, and
    #: TENSOR_2D_SURFACE.
    type: SymbolicConstant

    #: A sequence of Strings specifying the labels for each component of the value. The length
    #: of the sequence must match the type. If **type** = TENSOR, the default value is **name** with
    #: the suffixes ('11', '22', '33', '12', '13', '23'). If **type** = VECTOR, the default value
    #: is **name** with the suffixes ('1', '2', '3'). If **type** = SCALAR, the default value is an
    #: empty sequence.
    componentLabels: tuple[str, ...] = ()

    #: A sequence of SymbolicConstants specifying which invariants should be calculated for
    #: this field. An empty sequence indicates that no invariants are valid for this field.
    #: Possible values
    #: are:MAGNITUDEMISESTRESCAPRESSINV3MAX_PRINCIPALMID_PRINCIPALMIN_PRINCIPALMAX_INPLANE_PRINCIPALMIN_INPLANE_PRINCIPALOUTOFPLANE_PRINCIPALThe
    #: default value is an empty sequence.
    validInvariants: SymbolicConstant

    #: A Boolean specifying whether the field is an engineering tensor or not. Setting
    #: isEngineeringTensor to true makes a tensor field behave as a strain like quantity where
    #: the off-diagonal components of tensor are halved for invariants computation. This
    #: parameter applies only to tensor field outputs. The default value is OFF.
    isEngineeringTensor: Boolean = OFF

    @overload
    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        description: str,
        type: SymbolicConstant,
        componentLabels: tuple = (),
        validInvariants: SymbolicConstant | None = None,
        isEngineeringTensor: Boolean = OFF,
    ):
        """This method creates a FieldOutput object.

        .. note::
            This function can be accessed by::

                session.odbs[name].steps[name].frames[i].FieldOutput

        Parameters
        ----------
        name
            A String specifying the output variable name.
        description
            A String specifying the output variable. Colon (:) should not be used as a part of the
            field output description.
        type
            A SymbolicConstant specifying the output type. Possible values are SCALAR, VECTOR,
            TENSOR_3D_FULL, TENSOR_3D_PLANAR, TENSOR_3D_SURFACE, TENSOR_2D_PLANAR, and
            TENSOR_2D_SURFACE.
        componentLabels
            A sequence of Strings specifying the labels for each component of the value. The length
            of the sequence must match the type. If **type** = TENSOR, the default value is **name** with
            the suffixes ('11', '22', '33', '12', '13', '23'). If **type** = VECTOR, the default value
            is **name** with the suffixes ('1', '2', '3'). If **type** = SCALAR, the default value is an
            empty sequence.
        validInvariants
            A sequence of SymbolicConstants specifying which invariants should be calculated for
            this field. An empty sequence indicates that no invariants are valid for this field.
            Possible values
            are:MAGNITUDEMISESTRESCAPRESSINV3MAX_PRINCIPALMID_PRINCIPALMIN_PRINCIPALMAX_INPLANE_PRINCIPALMIN_INPLANE_PRINCIPALOUTOFPLANE_PRINCIPALThe
            default value is an empty sequence.
        isEngineeringTensor
            A Boolean specifying whether the field is an engineering tensor or not. Setting
            isEngineeringTensor to true makes a tensor field behave as a strain like quantity where
            the off-diagonal components of tensor are halved for invariants computation. This
            parameter applies only to tensor field outputs. The default value is OFF.

        Returns
        -------
        FieldOutput
            A FieldOutput object.
        """
        ...

    @overload
    @abaqus_method_doc
    def __init__(self, field: "FieldOutput", name: str = "", description: str = ""):
        """This method creates a FieldOutput object from an existing FieldOutput object of the same output
        database.

        .. note::
            This function can be accessed by::

                session.odbs[name].steps[name].frames[i].FieldOutput

        Parameters
        ----------
        field
            A FieldOutput object.
        name
            A String specifying the name of the FieldOutput object.
        description
            A String specifying the output variable. Colon (:) should not be used as a part of the
            field output description.

        Returns
        -------
        FieldOutput
            A FieldOutput object.
        """
        ...

    @abaqus_method_doc
    def __init__(self, *args, **kwargs): ...

    @overload
    def addData(
        self,
        position: SymbolicConstant,
        instance: OdbInstance,
        labels: tuple,
        data: tuple,
        sectionPoint: SectionPoint | None = None,
        localCoordSystem: tuple = (),
    ):
        """This method adds data to a FieldOutput object.

        Parameters
        ----------
        position
            A SymbolicConstant specifying the position of the output. Possible values are:

            - NODAL, specifying the values calculated at the nodes.
            - INTEGRATION_POINT, specifying the values calculated at the integration points.
            - ELEMENT_NODAL, specifying the values obtained by extrapolating results calculated at
              the integration points.
            - CENTROID, specifying the value at the centroid obtained by extrapolating results
              calculated at the integration points.
            - ELEMENT_FACE_INTEGRATION_POINT, specifying the values calculated at the element face
              integration points.
            - SURFACE_INTEGRATION_POINT, specifying the values calculated at the surface integration
              points. Selecting this value prompts the Visualization module to calculate the sum of
            the values at the ELEMENT_FACE_INTEGRATION_POINT position from multiple surfaces.
        instance
            An OdbInstance object specifying the namespace for labels.
        labels
            A sequence of Ints specifying the labels of the nodes or elements where the values in
            **data** are located. For better performance, the node or element labels are preferred to
            be sorted in ascending order and must be specified in the same order as the values
            provided for the **data** argument.
        data
            A sequence of sequences of Floats specifying the data values for the specified
            **position**, **instance**, and **labels**. The values must be given in the correct order.
            Element nodal data follow the order of nodal connectivity defined in the Abaqus
            documentation. Integration point data follow the order defined in the Abaqus
            documentation. Section point data for beams and shells follow the convention given in
            the Abaqus documentation. For more information, see the Abaqus Elements Guide. These
            data create FieldValue objects internally.
        sectionPoint
            A SectionPoint object specifying the location in the section. Although **sectionPoint** is
            an optional argument to the addData method, omitting the argument does have consequences
            for visualization. If you omit the argument when you are writing field output data for a
            shell or a beam, you cannot subsequently select the section point to display when you
            are displaying the field output data using the Visualization module.
        localCoordSystem
            The **localCoordSystem** parameter can be specified using either of the following:A
            sequence of sequences of Floats specifying the 3 x 3 matrix of direction cosines of the
            local coordinate system. This argument is available only for fields with type=TENSOR or
            VECTOR.A sequence of matrices of floats specifying the direction cosines of the local
            coordinates systems, where the sequence is the same length as **data**. If
            **localCoordSystem** is a matrix, a different local coordinate system applies to each data
            value.User supplied values of localCoordSystem are transposed before storing in the
            database.

        Raises
        ------
        odbException: Transformation not allowed for scalar data.
            The addData method throws many exceptions of type odbException. For example, if the
            local coordinate system is specified for scalar data
        """
        ...

    @overload
    @abaqus_method_doc
    def addData(self, field: "FieldOutput"):
        """This method adds the data from a field created using the getSubset method and mathematical operators
        to the database. The user must create a field to contain the new data and then use the addData method to
        assign the data from the fields.

        Parameters
        ----------
        field
            A FieldOutput object specifying the data to add.

        Raises
        ------
        odbException: Transformation not allowed for scalar data.
            The addData method throws many exceptions of type odbException. For example, if the
            local coordinate system is specified for scalar data
        """
        ...

    @overload
    @abaqus_method_doc
    def addData(
        self,
        position: SymbolicConstant,
        set: OdbSet,
        data: tuple,
        sectionPoint: SectionPoint | None = None,
        conjugateData: float | None = None,
    ):
        """This method adds data to a FieldOutput object.

        Parameters
        ----------
        position
            A SymbolicConstant specifying the position of the output. Possible values are:NODAL,
            specifying the values calculated at the nodes.INTEGRATION_POINT, specifying the values
            calculated at the integration points.ELEMENT_NODAL, specifying the values obtained by
            extrapolating results calculated at the integration points.CENTROID, specifying the
            value at the centroid obtained by extrapolating results calculated at the integration
            points.ELEMENT_FACE_INTEGRATION_POINT, specifying the values calculated at the element
            face integration points.SURFACE_INTEGRATION_POINT, specifying the values calculated at
            the surface integration points. Selecting this value prompts the Visualization module to
            calculate the sum of the values at the ELEMENT_FACE_INTEGRATION_POINT position from
            multiple surfaces.
        set
            An OdbSet object specifying the instance-level set defining the region for addData. The
            set must be defined in the same output database as the output database into which the
            new field output data is being written. For better performance, the node or element
            labels in the set are preferred to be sorted in ascending order and must be specified in
            the same order as the values provided for the **data** argument.
        data
            A sequence of sequences of Floats specifying the data values for the specified position
            and labels in the set. Each row of data provides the value at one unique position. The
            width of each row should match the number of required components for the data. The
            values must be given in the order that matches the ordering of labels in the set.The
            order of the element nodal data, integration point data, and section point data for
            beams and shells follows the conventions defined in the Abaqus Elements Guide.
        sectionPoint
            A SectionPoint object specifying the location in the section. Although*sectionPoint* is
            an optional argument to theaddData method, omitting the argument does have consequences
            for visualization. If you omit the argument when you are writing field output data for a
            shell or a beam, you cannot subsequently select the section point to display when you
            are displaying the field output data using the Visualization module.
        conjugateData
            An odb_SequenceSequenceFloat object specifying the imaginary data values for the
            specified position, instance, and labels. You must provide this data when you add
            complex fields to the output database. The order of the values follows the conventions
            defined in the Abaqus Elements Guide.

        Raises
        ------
        odbException
            Entities from multiple instances present in set, If you specify an odbSet containing entities from multiple instances
        dbException: Transformation not allowed for scalar data
            The addData method throws many exceptions of type odbException. For example, if the
            local coordinate system is specified for scalar data
        """
        ...

    @abaqus_method_doc
    def addData(self, *args, **kwargs): ...

    @overload
    def getScalarField(self, invariant: SymbolicConstant):
        """This method generates a scalar field containing the extracted component or calculated invariant
        values. The new field will hold values for the same nodes or elements as the parent field. Abaqus will
        perform this operation on only the real part of the FieldOutput object. The operation is not performed
        on the conjugate data (the imaginary portion of a complex result).

        Parameters
        ----------
        invariant
            A SymbolicConstant specifying the invariant. Possible values areMAGNITUDE, MISES,
            TRESCA, PRESS, INV3, MAX_PRINCIPAL, MID_PRINCIPAL, MIN_PRINCIPAL, MAX_INPLANE_PRINCIPAL,
            MIN_INPLANE_PRINCIPAL, and OUTOFPLANE_PRINCIPAL.

        Returns
        -------
        FieldOutput
            A FieldOutput object.
        """
        ...

    @overload
    @abaqus_method_doc
    def getScalarField(self, componentLabel: str):
        """This method generates a scalar field containing the extracted component or calculated invariant
        values. The new field will hold values for the same nodes or elements as the parent field. Abaqus will
        perform this operation on only the real part of the FieldOutput object. The operation is not performed
        on the conjugate data (the imaginary portion of a complex result).

        Parameters
        ----------
        componentLabel
            A String specifying the component label, such as “S11”.

        Returns
        -------
        FieldOutput
            A FieldOutput object.
        """
        ...

    @abaqus_method_doc
    def getScalarField(self, *args, **kwargs): ...

    @overload
    @abaqus_method_doc
    def getSubset(self, position: SymbolicConstant, readOnly: Boolean = OFF) -> "FieldOutput":
        """A FieldOutput object with a subset of the field values.

        Parameters
        ----------
        position
            **position**
            A SymbolicConstant specifying the position of the output in the element. Possible values
            are:

            - NODAL, specifying the values calculated at the nodes.
            - INTEGRATION_POINT, specifying the values calculated at the integration points.
            - ELEMENT_NODAL, specifying the values obtained by extrapolating results calculated at
              the integration points.
            - CENTROID, specifying the value at the centroid obtained by extrapolating results
              calculated at the integration points.
              If the requested field values are not found in the output database at the specified
              ELEMENT_NODAL or CENTROID positions, they are extrapolated from the field data at the
              INTEGRATION_POINT position for the entire field region. If the field values are found at
              the specified positions, only these field values are returned without any extrapolation.
              This could potentially be only for a subset of the field region, depending on the output
              request.
        readOnly
            A Boolean specifying whether the extrapolated data returned by this call is written to
            the output database. The default value is OFF.

        Returns
        -------
        FieldOutput
            A FieldOutput object.
        """
        ...

    @overload
    @abaqus_method_doc
    def getSubset(self, region: OdbSet) -> "FieldOutput":
        """A FieldOutput object with a subset of the field values.

        Parameters
        ----------
        region
            An OdbSet specifying the region for which to extract values. For better performance, the
            node or element labels in the sets are preferred to be sorted in ascending order.

        Returns
        -------
        FieldOutput
            A FieldOutput object.
        """
        ...

    @overload
    @abaqus_method_doc
    def getSubset(self, localCoordSystem: Sequence[Sequence[float]]) -> "FieldOutput":
        """A FieldOutput object with a subset of the field values.

        Parameters
        ----------
        localCoordSystem
            A sequence of sequences of Floats specifying the 3 x 3 matrix of direction cosines.
            Field values associated with the supplied coordinate system will be extracted.

        Returns
        -------
        FieldOutput
            A FieldOutput object.
        """
        ...

    @overload
    @abaqus_method_doc
    def getSubset(self, sectionPoint: SectionPoint) -> "FieldOutput":
        """A FieldOutput object with a subset of the field values.

        Parameters
        ----------
        sectionPoint
            A SectionPoint object.

        Returns
        -------
        FieldOutput
            A FieldOutput object.
        """
        ...

    @overload
    @abaqus_method_doc
    def getSubset(self, location: FieldLocation) -> "FieldOutput":
        """A FieldOutput object with a subset of the field values.

        Parameters
        ----------
        location
            A FieldLocation object.

        Returns
        -------
        FieldOutput
            A FieldOutput object.
        """
        ...

    @overload
    @abaqus_method_doc
    def getSubset(self, region: OdbMeshElement) -> "FieldOutput":
        """A FieldOutput object with a subset of the field values.

        Parameters
        ----------
        region
            An OdbMeshElement specifying the region for which to extract values.

        Returns
        -------
        FieldOutput
            A FieldOutput object.
        """
        ...

    @overload
    @abaqus_method_doc
    def getSubset(self, region: OdbMeshNode) -> "FieldOutput":
        """A FieldOutput object with a subset of the field values.

        Parameters
        ----------
        region
            An OdbMeshNode specifying the region for which to extract values.

        Returns
        -------
        FieldOutput
            A FieldOutput object.
        """
        ...

    @overload
    @abaqus_method_doc
    def getSubset(self, region: "FieldOutput") -> "FieldOutput":
        """A FieldOutput object with a subset of the field values.

        Parameters
        ----------
        region
            An OdbInstance specifying the region for which to extract values.

        Returns
        -------
        FieldOutput
            A FieldOutput object.
        """
        ...

    @overload
    @abaqus_method_doc
    def getSubset(self, elementType: str) -> "FieldOutput":
        """A FieldOutput object with a subset of the field values.

        Parameters
        ----------
        elementType
            A String specifying the element type for which to extract values. The string must
            correspond to a valid Abaqus element type.

        Returns
        -------
        FieldOutput
            A FieldOutput object.
        """
        ...

    @abaqus_method_doc
    def getSubset(self, *args, **kwargs) -> "FieldOutput":
        return FieldOutput("", "", C.SCALAR)

    @overload
    def getTransformedField(self, datumCsys: str, projected22Axis: int | None = None, projectionTol: str = ""):
        """This method generates a new vector or tensor field containing the transformed component values of the
        parent field. The new field will hold values for the same nodes or elements as the parent field. Results
        will be transformed based on the orientations specified by the input DatumCsys object. Abaqus will
        perform this operation on only the real part of the FieldOutput object. The operation is not performed
        on the conjugate data (the imaginary portion of a complex result).

        Parameters
        ----------
        datumCsys
            A valid DatumCsys object designating the coordinate system. Valid systems can be fixed
            or positioned with respect to nodes on the model and can be cartesian, cylindrical, or
            spherical.
        projected22Axis
            An Int specifying which axis of the coordinate system will be projected as the second
            component for local result orientations. Valid values are 1, 2, or 3; the default value
            is 2.
        projectionTol
            A Double specifying the minimum allowable angle (radians) between the specified
            projection axis and the element normal. The next axis will be used for projection if
            this tolerance test fails.

        Returns
        -------
        FieldOutput
            A FieldOutput object.

        Raises
        ------
        odbException
            Cannot apply transformation to field containing assembly level nodes, The getTransformedField method throws an exception if the field contains any assembly
            level nodes.
        """
        ...

    @overload
    @abaqus_method_doc
    def getTransformedField(
        self,
        datumCsys: str,
        deformationField: Union["FieldOutput", None] = None,
        projected22Axis: int | None = None,
        projectionTol: str = "",
    ):
        """This method generates a new vector or tensor field containing the transformed component values of the
        parent field. The new field will hold values for the same nodes or elements as the parent field. Results
        will be transformed based on the orientations specified by the input DatumCsys object. Abaqus will
        perform this operation on only the real part of the FieldOutput object. The operation is not performed
        on the conjugate data (the imaginary portion of a complex result).

        Parameters
        ----------
        datumCsys
            A valid DatumCsys object designating the coordinate system. Valid systems can be fixed
            or positioned with respect to nodes on the model and can be cartesian, cylindrical, or
            spherical.
        deformationField
            A FieldOutput object specifying the nodal displacement vectors required by moving
            coordinate systems to determine instantaneous configurations.
        projected22Axis
            An Int specifying which axis of the coordinate system will be projected as the second
            component for local result orientations. Valid values are 1, 2, or 3; the default value
            is 2.
        projectionTol
            A Double specifying the minimum allowable angle (radians) between the specified
            projection axis and the element normal. The next axis will be used for projection if
            this tolerance test fails.

        Returns
        -------
        FieldOutput
            A FieldOutput object.

        Raises
        ------
        odbException
            Cannot apply transformation to field containing assembly level nodes, The getTransformedField method throws an exception if the field contains any assembly
            level nodes.
        """
        ...

    @overload
    @abaqus_method_doc
    def getTransformedField(
        self,
        datumCsys: str,
        deformationField: Union["FieldOutput", None] = None,
        rotationField: Union["FieldOutput", None] = None,
        projected22Axis: int | None = None,
        projectionTol: str = "",
    ):
        """This method generates a new vector or tensor field containing the transformed component values of the
        parent field. The new field will hold values for the same nodes or elements as the parent field. Results
        will be transformed based on the orientations specified by the input DatumCsys object. Abaqus will
        perform this operation on only the real part of the FieldOutput object. The operation is not performed
        on the conjugate data (the imaginary portion of a complex result).

        Parameters
        ----------
        datumCsys
            A valid DatumCsys object designating the coordinate system. Valid systems can be fixed
            or positioned with respect to nodes on the model and can be cartesian, cylindrical, or
            spherical.
        deformationField
            A FieldOutput object specifying the nodal displacement vectors required by moving
            coordinate systems to determine instantaneous configurations.
        rotationField
            A FieldOutput object specifying the nodal rotational displacement vectors required by
            moving coordinate systems that follow a 6-dof node, to determine instantaneous
            configurations.
        projected22Axis
            An Int specifying which axis of the coordinate system will be projected as the second
            component for local result orientations. Valid values are 1, 2, or 3; the default value
            is 2.
        projectionTol
            A Double specifying the minimum allowable angle (radians) between the specified
            projection axis and the element normal. The next axis will be used for projection if
            this tolerance test fails.

        Returns
        -------
        FieldOutput
            A FieldOutput object.

        Raises
        ------
        odbException
            Cannot apply transformation to field containing assembly level nodes, The getTransformedField method throws an exception if the field contains any assembly
            level nodes.
        """
        ...

    @abaqus_method_doc
    def getTransformedField(self, *args, **kwargs): ...

    def getConnectorFieldXformedToNodeA(self, deformationField: Union["FieldOutput", None] = None):
        """This method generates a new vector field containing the transformed component values of the parent
        connector field to the node A coordinate system. The new field will hold values for the same connector
        elements as the parent field. Some connection types such as Axial, Link, Slip Ring, and Radial Thrust
        require that the deformationField be specified.

        Parameters
        ----------
        deformationField
            A FieldOutput object specifying the nodal displacement vectors required by moving
            coordinate systems to determine instantaneous configurations.

        Returns
        -------
        FieldOutput
            A FieldOutput object.

        Raises
        ------
        odbException: Deformation field is required for transforming this connector field.
            The getConnectorFieldXformedToNodeA method throws an exception if the field requires a
            deformationField and the argument is not supplied.
        """
        ...

    @abaqus_method_doc
    def setComponentLabels(self, componentLabels: tuple):
        """This method sets the component labels for the FieldOutput object.

        Parameters
        ----------
        componentLabels
            A sequence of Strings specifying the labels for each component of the value. The length
            of the sequence must match the type. If **type** = TENSOR, the default value is **name** with
            the suffixes ('11', '22', '33', '12', '13', '23'). If **type** = VECTOR, the default value
            is **name** with the suffixes ('1', '2', '3'). If **type** = SCALAR, the default value is an
            empty sequence.
        """
        ...

    @abaqus_method_doc
    def setDataType(
        self,
        type: Literal[
            C.TENSOR_2D_SURFACE,
            C.TENSOR_3D_SURFACE,
            C.VECTOR,
            C.TENSOR_3D_FULL,
            C.SCALAR,
            C.TENSOR_2D_PLANAR,
            C.TENSOR_3D_PLANAR,
        ],
    ):
        """This method sets the data type of a FieldOutput object.

        Parameters
        ----------
        type
            A SymbolicConstant specifying the output type. Possible values are SCALAR, VECTOR,
            TENSOR_3D_FULL, TENSOR_3D_PLANAR, TENSOR_3D_SURFACE, TENSOR_2D_PLANAR, and
            TENSOR_2D_SURFACE.
        """
        ...

    @abaqus_method_doc
    def setValidInvariants(
        self,
        validInvariants: Literal[
            C.MISES,
            C.MAX_PRINCIPAL,
            C.MIN_PRINCIPAL,
            C.MID_PRINCIPAL,
            C.MAGNITUDE,
            C.OUTOFPLANE_PRINCIPAL,
            C.TRESCA,
            C.MIN_INPLANE_PRINCIPAL,
            C.MAX_INPLANE_PRINCIPAL,
            C.INV3,
            C.PRESS,
        ],
    ):
        """This method sets the invariants valid for the FieldOutput object.

        Parameters
        ----------
        validInvariants
            A sequence of SymbolicConstants specifying which invariants should be calculated for
            this field. An empty sequence indicates that no invariants are valid for this field.
            Possible values are:

            - MAGNITUDE
            - MISES
            - TRESCA
            - PRESS
            - INV3
            - MAX_PRINCIPAL
            - MID_PRINCIPAL
            - MIN_PRINCIPAL
            - MAX_INPLANE_PRINCIPAL
            - MIN_INPLANE_PRINCIPAL
            - OUTOFPLANE_PRINCIPAL

            The default value is an empty sequence.
        """
        ...



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..UtilityAndView.abaqusConstants import (
    DEFAULT_MODEL,
    ON,
    Boolean,
    SymbolicConstant,
)
from .Leaf import Leaf


@abaqus_class_doc
class LeafFromSurfaceVarRange(Leaf):
    """The LeafFromSurfaceVarRange object can be used whenever a Leaf object is expected as an argument. Leaf
    objects are used to specify the items in a display group. Leaf objects are constructed as temporary objects,
    which are then used as arguments to DisplayGroup commands. The LeafFromSurfaceVarRange object is derived
    from the Leaf object.

    .. note::
        This object can be accessed by::

            import displayGroupOdbToolset
    """

    #: A SymbolicConstant specifying the leaf type. Possible values are EMPTY_LEAF,
    #: DEFAULT_MODEL, ALL_ELEMENTS, ALL_NODES, and ALL_SURFACES.
    leafType: SymbolicConstant

    #: A Float specifying the minimum value for the variable range. The default value is
    #: −3.40282346639E38.
    minimumRange: float | None = None

    #: A Float specifying the maximum value for the variable range. The default value is
    #: 3.40282346639e+038.
    maximumRange: float = 3

    #: A Boolean specifying the method used to evaluate the range. If **insideRange** = ON, the
    #: range falls inside the specified minimum and maximum values. The default value is ON.
    insideRange: Boolean = ON

    @abaqus_method_doc
    def __init__(
        self,
        minimumRange: float | None = None,
        maximumRange: float = 3,
        insideRange: Boolean = ON,
    ):
        """This method creates a Leaf object from surfaces with values lying in a variable range.

        .. note::
            This function can be accessed by::

                LeafFromSurfaceVarRange

        Parameters
        ----------
        minimumRange
            A Float specifying the minimum value for the variable range. The default value is
            −3.40282346639E38.
        maximumRange
            A Float specifying the maximum value for the variable range. The default value is
            3.40282346639e+038.
        insideRange
            A Boolean specifying the method used to evaluate the range. If **insideRange** = ON, the
            range falls inside the specified minimum and maximum values. The default value is ON.

        Returns
        -------
        LeafFromSurfaceVarRange
            A LeafFromSurfaceVarRange object.
        """
        super().__init__(DEFAULT_MODEL)
